/**
 * 
 */
package org.processmining.plugins.petrinet.analysis;

import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.processmining.plugins.petrinet.analysis.PNPluginSelectionPanel.AnalysisTypes;

/**
 * @author Arya Adriansyah
 * @email a.adriansyah@tue.nl
 * @version May 31, 2010
 */
public class DependencyFactory {
	public static Map<AnalysisTypes, Set<AnalysisTypes>> getStructuralPluginDependencies() {
		// create dependencies of structural plugins
		Map<AnalysisTypes, Set<AnalysisTypes>> dependencies = new HashMap<AnalysisTypes, Set<AnalysisTypes>>();

		// invariants identification is independent
		dependencies.put(AnalysisTypes.PLACEINVARIANT, Collections.<AnalysisTypes>emptySet());
		dependencies.put(AnalysisTypes.TRANSITIONINVARIANT, Collections.<AnalysisTypes>emptySet());

		// handles identification is independent
		dependencies.put(AnalysisTypes.TPHANDLES, Collections.<AnalysisTypes>emptySet());
		dependencies.put(AnalysisTypes.PTHANDLES, Collections.<AnalysisTypes>emptySet());

		// s component identification depends on place invariant
		Set<AnalysisTypes> sComDep = new HashSet<AnalysisTypes>();
		sComDep.add(AnalysisTypes.PLACEINVARIANT);
		dependencies.put(AnalysisTypes.SCOMPONENTSET, sComDep);

		// t component identification depends on transition invariant
		Set<AnalysisTypes> tComDep = new HashSet<AnalysisTypes>();
		tComDep.add(AnalysisTypes.TRANSITIONINVARIANT);
		dependencies.put(AnalysisTypes.TCOMPONENTSET, tComDep);

		// siphons identification is independent
		dependencies.put(AnalysisTypes.SIPHONSET, Collections.<AnalysisTypes>emptySet());

		// traps identification is independent
		dependencies.put(AnalysisTypes.TRAPSET, Collections.<AnalysisTypes>emptySet());

		// free choice identification is independent
		dependencies.put(AnalysisTypes.FREECHOICE, Collections.<AnalysisTypes>emptySet());
		dependencies.put(AnalysisTypes.EXTFREECHOICE, Collections.<AnalysisTypes>emptySet());
		dependencies.put(AnalysisTypes.NONFREECHOICECLUS, Collections.<AnalysisTypes>emptySet());
		dependencies.put(AnalysisTypes.NONFREEEXTCHOICECLUS, Collections.<AnalysisTypes>emptySet());
		
		// relaxed soundness identification is independent
//		dependencies.put(AnalysisTypes.RELAXEDSOUNDESS, Collections.<AnalysisTypes>emptySet());
//		dependencies.put(AnalysisTypes.UNFIREABLETRANSITIONRELAXEDSOUNDNESS, Collections.<AnalysisTypes>emptySet());

		Set<AnalysisTypes> all = new HashSet<AnalysisTypes>();
		for (AnalysisTypes type : PNPluginSelectionPanel.AnalysisTypes.values()) {
			if (type != AnalysisTypes.ALLSTRUCTURAL && !type.isBehavioral()) {
				all.add(type);
			}
		}
		dependencies.put(AnalysisTypes.ALLSTRUCTURAL, all);
		
		return dependencies;
	}

	public static Map<AnalysisTypes, Set<AnalysisTypes>> getBehavioralPluginDependencies() {
		// create dependencies of structural plugins
		Map<AnalysisTypes, Set<AnalysisTypes>> dependencies = new HashMap<AnalysisTypes, Set<AnalysisTypes>>();

		dependencies.put(AnalysisTypes.BOUNDEDNESSINFO, Collections.<AnalysisTypes>emptySet()); // boundedness

		dependencies.put(AnalysisTypes.BOUNDEDNESSSEQ, Collections.<AnalysisTypes>emptySet()); // boundedness

		dependencies.put(AnalysisTypes.UNBOUNDEDPLACES, Collections.<AnalysisTypes>emptySet()); // boundedness

		dependencies.put(AnalysisTypes.DEADTRANSITIONS, Collections.<AnalysisTypes>emptySet()); // dead transition

		dependencies.put(AnalysisTypes.HOMEMARKINGSET, Collections.<AnalysisTypes>emptySet()); // home marking

		dependencies.put(AnalysisTypes.LIVENESSINFO, Collections.<AnalysisTypes>emptySet()); // liveness

		dependencies.put(AnalysisTypes.LIVENESSTRANS, Collections.<AnalysisTypes>emptySet()); // liveness

		// relaxed soundness identification is independent
//		dependencies.put(AnalysisTypes.RELAXEDSOUNDESS, Collections.<AnalysisTypes>emptySet());
//		dependencies.put(AnalysisTypes.UNFIREABLETRANSITIONRELAXEDSOUNDNESS, Collections.<AnalysisTypes>emptySet());

		Set<AnalysisTypes> all = new HashSet<AnalysisTypes>();
		for (AnalysisTypes type : PNPluginSelectionPanel.AnalysisTypes.values()) {
			if (type != AnalysisTypes.ALLBEHAVIORAL && type.isBehavioral()) {
				all.add(type);
			}
		}
		dependencies.put(AnalysisTypes.ALLBEHAVIORAL, all);

		return dependencies;
	}
}
