/**
 * 
 */
package org.processmining.plugins.petrinet.analysis;

import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginCategory;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.plugins.petrinet.analysis.PNPluginSelectionPanel.AnalysisTypes;

/**
 * @author Arya Adriansyah
 * @email a.adriansyah@tue.nl
 * @version May 31, 2010
 */
@Plugin(name = "Analyze Structural Property of Petri net", level = PluginLevel.PeerReviewed, categories = { PluginCategory.Analytics }, returnLabels = { "Analyzed net" }, returnTypes = { Petrinet.class }, parameterLabels = {
		"Net", "Plugins to be executed" }, userAccessible = true/*, mostSignificantResult = -1*/)
public class PNStructuralAnalysis {

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "A. Adriansyah, D. Fahland", email = "d.fahland@tue.nl", pack = "PNAnalysis")
	@PluginVariant(variantLabel = "Analyze Structural Property of Petri Net", requiredParameterLabels = { 0 })
	public Petrinet analyzeStructuralProperty(UIPluginContext context, Petrinet net) {
		Set<AnalysisTypes> available = new HashSet<AnalysisTypes>();

		for (AnalysisTypes type : PNPluginSelectionPanel.AnalysisTypes.values()) {
			try {
				if (type != AnalysisTypes.ALLSTRUCTURAL && !type.isBehavioral()) {
					context.getConnectionManager().getFirstConnection(type.getConnectionType(), context, net);
					available.add(type);
				}
			} catch (ConnectionCannotBeObtained e) {
				// Don't care
			}
		}

		Map<AnalysisTypes, Set<AnalysisTypes>> dependencies = DependencyFactory.getStructuralPluginDependencies();

		PNPluginSelectionPanel panel = new PNPluginSelectionPanel(available, dependencies, "Select Analysis Plugins",
				"To select a plugin, click the name of the plugin on the list below. To deselect it, click again the name of the plugin. <br/>"
						+ "If a plug-in depends on other plugin(s), the plugins will automatically be selected.");

		InteractionResult result = context.showConfiguration("Structural Analysis Plug-in", panel);
		if (result != InteractionResult.CONTINUE) {
			context.getFutureResult(0).cancel(true);
			return net;
		}

		Set<AnalysisTypes> propertiesToBeCalculated = panel.getPropertiesToBeCalculated();

		if (propertiesToBeCalculated.size() > 0) {
			// calculate result
			analyzeStructuralProperty(context, net, propertiesToBeCalculated.toArray(new AnalysisTypes[0]));

		}
		
		return net;
	}

	@PluginVariant(variantLabel = "Analyze Structural Property of Petri Net", requiredParameterLabels = { 0, 1 })
	public Petrinet analyzeStructuralProperty(PluginContext context, Petrinet net, AnalysisTypes[] pluginToBeExecuted) {

		for (AnalysisTypes type : pluginToBeExecuted) {
			if (type != AnalysisTypes.ALLSTRUCTURAL) {
				try {
					context.tryToFindOrConstructFirstObject(type.getObjectType(), type.getConnectionType(),
							type.getRole(), net);
				} catch (ConnectionCannotBeObtained e) {
					context.log(e);
				}
			}
		}
		return net;
	}

}
