/**
 * 
 */
package org.processmining.plugins.petrinet.analysis;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.processmining.plugins.petrinet.analysis.PNPluginSelectionPanel.AnalysisTypes;

import com.fluxicon.slickerbox.components.GradientPanel;

/**
 * @author Arya Adriansyah, Dirk Fahland
 * @email a.adriansyah@tue.nl, d.fahland@tue.nl
 * @version May 31, 2010
 */
public class SlickerPluginDependencyConfiguration extends JPanel {

	private static final long serialVersionUID = 2392584567665617821L;

	protected static Color colorText = new Color(255, 255, 255, 160);
	protected static Color colorGreen1 = new Color(10, 90, 1);
	protected static Color colorGreen2 = new Color(20, 140, 20);
	protected static Color colorOrange1 = new Color(90, 60, 10);
	protected static Color colorOrange2 = new Color(140, 100, 20);
	protected static Color colorRed1 = new Color(90, 10, 10);
	protected static Color colorRed2 = new Color(140, 20, 20);
	protected static Color colorGray1 = new Color(40, 40, 40);
	protected static Color colorGray2 = new Color(100, 100, 100);

	public enum SelectionAction {
		DONOTCALCULATE, CALCULATE, REQUIRED, EXIST;
	}

	protected PluginConfigurationItem[] configurationItems;
	protected ChangeListener updateListener = null;
	protected Map<AnalysisTypes, Set<AnalysisTypes>> dependencies = null;

	public SlickerPluginDependencyConfiguration(Set<AnalysisTypes> calculatedProp,
			Map<AnalysisTypes, Set<AnalysisTypes>> dependencies) {
		setBackground(new Color(60, 60, 60));
		setMinimumSize(new Dimension(70, 40));
		setMaximumSize(new Dimension(1000, 5000));
		setBorder(BorderFactory.createEmptyBorder());
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		configurationItems = new PluginConfigurationItem[dependencies.size()];
		int height = 0;
		
		// sort analysis types by name
		AnalysisTypes[] propertiesSorted = dependencies.keySet().toArray(new AnalysisTypes[dependencies.size()]);
		Arrays.sort(propertiesSorted, new AnalysisTypes.AnalysisTypesComp());
		
		int i = 0;
		for (AnalysisTypes property : propertiesSorted) {
			if (calculatedProp.contains(property)) {
				configurationItems[i] = new PluginConfigurationItem(property, SelectionAction.EXIST);
			} else {
				configurationItems[i] = new PluginConfigurationItem(property, SelectionAction.DONOTCALCULATE);
			}
			this.add(configurationItems[i]);
			height += configurationItems[i].getPreferredSize().height + 2;
			i++;
		}

		this.dependencies = dependencies;
		this.add(Box.createVerticalGlue());
		setPreferredSize(new Dimension(120, height));
		revalidate();
	}

	public Set<AnalysisTypes> getPluginsToBeCalculated() {
		Set<AnalysisTypes> newSet = new HashSet<AnalysisTypes>();
		for (int i = 0; i < configurationItems.length; i++) {
			if ((configurationItems[i].getAction() == SelectionAction.CALCULATE)
					|| (configurationItems[i].getAction() == SelectionAction.REQUIRED)) {
				newSet.add(configurationItems[i].getObject());
			}
		}
		return newSet;
	}

	protected class PluginConfigurationItem extends GradientPanel {
		private static final long serialVersionUID = -7596401396201481445L;
		protected AnalysisTypes object;
		protected SelectionAction action;
		protected JLabel actionLabel;
		protected JLabel nameLabel;

		public PluginConfigurationItem(AnalysisTypes object, SelectionAction act) {
			super(colorGreen2, colorGreen1);
			setBorder(BorderFactory.createEmptyBorder(4, 10, 4, 10));
			setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
			this.object = object;
			nameLabel = new JLabel(object.getName());
			nameLabel.setOpaque(false);
			nameLabel.setForeground(colorText);
			nameLabel.setVerticalAlignment(SwingConstants.CENTER);
			nameLabel.setFont(nameLabel.getFont().deriveFont(13f));
			actionLabel = new JLabel("change_me");
			actionLabel.setOpaque(false);
			actionLabel.setForeground(colorText);
			actionLabel.setVerticalAlignment(SwingConstants.CENTER);
			actionLabel.setFont(actionLabel.getFont().deriveFont(12f).deriveFont(Font.ITALIC));

			setAction(act);

			this.add(nameLabel);
			this.add(Box.createHorizontalGlue());
			this.add(actionLabel);
			addMouseListener(new MouseListener() {
				public void mouseClicked(MouseEvent arg0) {
					if (action == SelectionAction.DONOTCALCULATE) {
						setAction(SelectionAction.CALCULATE);
					} else if (action == SelectionAction.REQUIRED) {
						setAction(SelectionAction.CALCULATE);
					} else if (action == SelectionAction.CALCULATE) {
						setAction(SelectionAction.DONOTCALCULATE);
					}

					updateActions();

					// notify update listener
					if (updateListener != null) {
						updateListener.stateChanged(new ChangeEvent(this));
					}
				}

				public void mouseEntered(MouseEvent arg0) { /* ignore */
				}

				public void mouseExited(MouseEvent arg0) { /* ignore */
				}

				public void mousePressed(MouseEvent arg0) { /* ignore */
				}

				public void mouseReleased(MouseEvent arg0) { /* ignore */
				}
			});
		}

		public void updateActions() {
			Set<Object> areRequired = new HashSet<Object>();

			// identify which plugins are required
			for (int i = 0; i < configurationItems.length; i++) {
				if (configurationItems[i].getAction() == SelectionAction.CALCULATE) {
					if (dependencies.get(configurationItems[i].getObject()) != null) {
						areRequired.addAll(dependencies.get(configurationItems[i].getObject()));
					}
				}
			}

			for (int i = 0; i < configurationItems.length; i++) {
				if (configurationItems[i].getAction() != SelectionAction.EXIST) {
					if (areRequired.contains(configurationItems[i].getObject())) {
						if (configurationItems[i].getAction() != SelectionAction.CALCULATE) {
							configurationItems[i].setAction(SelectionAction.REQUIRED);
						}
					} else {
						if (configurationItems[i].getAction() != SelectionAction.CALCULATE) {
							configurationItems[i].setAction(SelectionAction.DONOTCALCULATE);
						}
					}
				}
			}
		}

		public SelectionAction getAction() {
			return action;
		}

		public AnalysisTypes getObject() {
			return object;
		}

		public void setAction(SelectionAction action) {
			this.action = action;
			if (action == SelectionAction.CALCULATE) {
				actionLabel.setText("(selected)");
				super.setColors(colorGreen2, colorGreen1);
			} else if (action == SelectionAction.REQUIRED) {
				actionLabel.setText("(selected due to dependency of selected plugin(s))");
				super.setColors(colorOrange2, colorOrange1);
			} else if (action == SelectionAction.EXIST) {
				actionLabel.setText("(result(s) of the plugin already exist)");
				super.setColors(colorRed2, colorRed1);
			} else if (action == SelectionAction.DONOTCALCULATE) {
				actionLabel.setText("(not selected)");
				super.setColors(colorGray2, colorGray1);
			}
		}
	}
}
